import os
import pathlib
import inspect
import flatdict
import itertools
import ninja.ninja_syntax


import pyhelios
import pyhelios.exe
import pyhelios.toml
import pyhelios.template


class GNinja(ninja.ninja_syntax.Writer):
    def __init__(self, output):
        ninja.ninja_syntax.Writer.__init__(self, output)
        template = os.path.dirname(inspect.getfile(pyhelios))
        template = os.path.join(template, "share", "templates", "build.ninja")
        with open(template, "r") as fid:
            rules = fid.read()
        self.output.write(rules)


def parametric_study(template, toml, output_dir):
    cases_list = []
    data = flatdict.FlatDict(pyhelios.toml.read(toml), delimiter="_")
    cases = list(itertools.product(*data.values()))
    for c in cases:
        d = dict(zip(data.keys(), list(str(item) for item in c)))
        d["parameters_case_name"] = "_".join(d.values())
        d["parameters_output"] = os.path.join(output_dir, d["parameters_case_name"])
        pyhelios.exe.mkdir(d["parameters_output"])
        d["parameters_output"] = os.path.join(
            d["parameters_output"], os.path.basename(template)
        )
        pyhelios.template.render(template, d["parameters_output"], d)
        cases_list.append(d)
    return cases_list



import os
import jinja2


def get_jinja_latex_template(template):
    options = dict()
    options["block_start_string"] = "\BLOCK{"
    options["block_end_string"] = "}"
    options["variable_start_string"] = "\VAR{"
    options["variable_end_string"] = "}"
    options["comment_start_string"] = "\#{"
    options["comment_end_string"] = "}"
    options["line_statement_prefix"] = "%%"
    options["line_comment_prefix"] = "%#"
    options["trim_blocks"] = True
    options["autoescape"] = False
    if os.path.isfile(template):
        dirname = os.path.dirname(template)
        basename = os.path.basename(template)
        options["loader"] = jinja2.FileSystemLoader(dirname)
        env = jinja2.Environment(**options)
        return env.get_template(basename)
    else:
        env = jinja2.Environment(**options)
        return env.from_string(template)


def render(template, output, data):
    options = {}
    options["trim_blocks"] = True
    options["autoescape"] = False
    dirname = os.path.dirname(template)
    basename = os.path.basename(template)
    options["loader"] = jinja2.FileSystemLoader(dirname)
    tmp = jinja2.Environment(**options).get_template(basename)
    with open(output, "w") as fid:
        fid.write(tmp.render(data))



import os
import numpy
import inspect

import pyhelios
import pyhelios.toml

from pyhelios.exe import exist, call_latex
from pyhelios.template import get_jinja_latex_template

NUMBER_MAJOR_TICK = 6
NUMBER_MINOR_TICK = 10
COLORS = {
    "b": "blue",
    "n": "orange",
    "r": "red",
    "l": "lightblue",
    "g": "green",
    "y": "yellow",
    "p": "purple",
    "i": "pink",
    "a": "brown",
    "e": "gray",
    "k": "black",
}
LINES = {"-": "solid", "--": "dashed", "m": "only marks", "..": "dotted"}
MARKERS = {
    "o": "mark = *",
    "+": "mark = +",
    "s": "mark = square*",
    "d": "mark = diamond*",
    "t": "mark = triangle*",
    "ow": "mark = *, mark options = {style = solid, fill = white}",
    "sw": "mark = square*, mark options = {style = solid, fill = white}",
    "dw": "mark = diamond*, mark options = {style = solid, fill = white}",
    "tw": "mark = triangle*, mark options = {style = solid, fill = white}",
}
DEFAULT_STYLES = ["k-", "r-ow", "b-sw", "g-o", "a-s"]


def logn(n, x):
    if n == 2:
        return numpy.log2(x)
    elif n == 10:
        return numpy.log10(x)
    else:
        return numpy.log(x) / numpy.log(n)


class Plot:
    def __init__(self, CfgFn, TmpFn, OutputDir="."):
        self.cfg_fn = CfgFn
        self.tmp_fn = TmpFn
        OutputDir = "" if OutputDir == "." else OutputDir
        self.cfg_abspath = os.path.dirname(os.path.abspath(self.cfg_fn))
        # --- Get TOML configuration.
        self.cfg = pyhelios.toml.read(CfgFn)
        pyhelios.logger.log("INFO", f"Reading {CfgFn}")
        # --- Get template.
        if not TmpFn:
            TmpFn = os.path.dirname(inspect.getfile(pyhelios))
            TmpFn = os.path.join(TmpFn, "share", "templates", "plot.tex")
        self.tmp = get_jinja_latex_template(TmpFn)
        pyhelios.logger.log("INFO", f"Reading {TmpFn}")
        # --- Loop over each figure contained in the TOML file.
        for fig in self.cfg["plot"]:
            self.opt = {"fig": ".".join([fig, self.cfg["plot"][fig]["format"]])}
            # --- Define output file name.
            self.out = os.path.join(OutputDir, self.opt["fig"])
            # --- Add keys coming from cfg file.
            self.opt.update(self.cfg["plot"][fig])
            # --- Parse data.
            self.opt["legend"] = True
            for l in self.opt["lines"]:
                data = l["data"]
                l["data"] = os.path.join(self.cfg_abspath, data[0])
                l["x"] = int(data[1])
                l["y"] = int(data[2])
                if l["label"] == None:
                    self.opt["legend"] = False
            # --- Log axis.
            for axis in ["x", "y"]:
                self.opt[axis + "log"] = False
                if "axis" in self.opt:
                    if "log" + axis in self.opt["axis"]:
                        self.opt[axis + "log"] = True
            # --- Define range and ticks.
            for axis in ["x", "y"]:
                self.get_range(axis, self.opt[axis + "log"])
                self.get_tick(axis, self.opt[axis + "log"])
                self.get_minor_tick(axis)
            # --- Parse style.
            self.parse_style()
            # --- Render LaTeX file.
            tex = os.path.splitext(self.out)[0] + ".tex"
            if "pdf" in os.path.splitext(self.out)[1]:
                self.opt["standalone"] = True
            with open(tex, "w") as fid:
                fid.write(self.tmp.render(**self.opt))
            level = "INFO" if exist(tex) else "ERROR"
            pyhelios.logger.log(level, f"Writing {tex}")
            if "pdf" in os.path.splitext(self.out)[1]:
                call_latex(os.path.basename(tex), OutputDir)
                level = "INFO" if exist(self.out) else "ERROR"
                pyhelios.logger.log(level, f"Writing {self.out}")

    def parse_style(self):
        for il, l in enumerate(self.opt["lines"]):
            if "style" not in l:
                l["style"] = DEFAULT_STYLES[il]
            for k, v in COLORS.items():
                if k in l["style"]:
                    l["color"] = v
            for k, v in LINES.items():
                if k in l["style"]:
                    l["line"] = v
            for k, v in MARKERS.items():
                if k in l["style"]:
                    l["marker"] = v

    def get_range(self, axis, log):
        if axis + "min" not in self.opt:
            self.opt[axis + "min"] = float("+inf")
            for l in self.opt["lines"]:
                M = numpy.loadtxt(l["data"])
                m = numpy.amin(M[:, l[axis]])
                self.opt[axis + "min"] = min(self.opt[axis + "min"], m)
        if axis + "max" not in self.opt:
            self.opt[axis + "max"] = float("-inf")
            for l in self.opt["lines"]:
                M = numpy.loadtxt(l["data"])
                m = numpy.amax(M[:, l[axis]])
                self.opt[axis + "max"] = max(self.opt[axis + "max"], m)
        if log:
            for ext in ["min", "max"]:
                self.opt[axis + ext] = logn(10, self.opt[axis + ext])
        delta = self.opt[axis + "max"] - self.opt[axis + "min"]
        for i, ext in enumerate(["max", "min"]):
            self.opt[axis + ext + "ext"] = self.opt[axis + ext]
            self.opt[axis + ext + "ext"] += (-1) ** i * 0.05 * delta
            self.opt[axis + ext + "eps"] = self.opt[axis + ext]
            self.opt[axis + ext + "eps"] += (-1) ** i * 1.0e-3 * delta
        if log:
            for ext in ["minext", "maxext"]:
                self.opt[axis + ext] = 10.0 ** (self.opt[axis + ext])

    def get_tick(self, axis, log):
        if axis + "tick" not in self.opt:
            m = self.opt[axis + "min"]
            M = self.opt[axis + "max"]
            n = NUMBER_MAJOR_TICK
            if log:
                self.opt[axis + "tick"] = numpy.logspace(m, M, n)
            else:
                self.opt[axis + "tick"] = numpy.linspace(m, M, n)

    def get_minor_tick(self, axis):
        key = "minor" + axis + "tick"
        if key not in self.opt:
            siz = len(self.opt[axis + "tick"]) - 1
            self.opt[key] = numpy.zeros(siz * NUMBER_MINOR_TICK)
            for it, t in enumerate(self.opt[axis + "tick"][0:-1]):
                m = self.opt[axis + "tick"][it]
                M = self.opt[axis + "tick"][it + 1]
                n = NUMBER_MINOR_TICK
                beg = it * NUMBER_MINOR_TICK
                end = beg + NUMBER_MINOR_TICK
                self.opt[key][beg:end] = numpy.linspace(m, M, n + 1)[0:-1]



import os
import glob
import errno
import shlex
import shutil
import pathlib
import subprocess


def run(cmd, stdin="", directory=".", shell=False):
    opt = {}
    opt["stdin"] = subprocess.PIPE
    opt["stdout"] = subprocess.PIPE
    opt["stderr"] = subprocess.PIPE
    opt["shell"] = shell
    current = pathlib.Path.cwd()
    working = pathlib.Path(directory)
    os.chdir(working)
    if shell:
        p = subprocess.Popen(cmd, **opt)
    else:
        p = subprocess.Popen(shlex.split(cmd, posix=False), **opt)
    stdout, stderr = p.communicate(input=stdin.encode())
    os.chdir(current)
    return [stdout.decode(), stderr.decode()]


def call_latex(tex, directory="."):
    run(f"pdflatex -interaction=batchmode {tex}", directory=directory)
    current = pathlib.Path.cwd()
    working = pathlib.Path(directory)
    os.chdir(working)
    for e in [".aux", ".log"]:
        os.remove(os.path.splitext(tex)[0] + e)
    os.chdir(current)


def mkdir(directory, mode=0o755):
    p = pathlib.Path(directory)
    p.mkdir(mode=mode, parents=True, exist_ok=True)
    os.chmod(directory, mode)


def cp(src, dst):
    try:
        shutil.copytree(src, dst)
    except OSError as e:
        if e.errno == errno.ENOTDIR:
            shutil.copy2(src, dst)


def rm(obj):
    for iobj in glob.glob(str(obj)):
        if os.path.isfile(iobj):
            os.remove(iobj)
        if os.path.isdir(iobj):
            shutil.rmtree(iobj)


def keep(obj):
    for iobj in glob.glob("*"):
        if iobj not in obj:
            rm(iobj)


def exist(obj):
    return os.path.isfile(obj)



from pygments import highlight
from pygments.style import Style
from pygments.lexers.compiled import CLexer
from pygments.formatters import LatexFormatter
from pygments.token import (
    Keyword,
    Name,
    Comment,
    String,
    Error,
    Literal,
    Number,
    Operator,
    Other,
    Punctuation,
    Text,
    Generic,
    Whitespace,
)


def pygments_latex_defines():
    return LatexFormatter(style=HeliosPygments).get_style_defs()


def pygments_latex_render(code):
    lex = HeliosCLexer()
    options = [
        "breaklines",
        "mathescape",
        "fontfamily = tt",
        "breaksymbolleft = \\raisebox{0.8ex}{\small\\reflectbox{\carriagereturn}}",
        "breaksymbolright = \small\carriagereturn",
    ]
    formatter = LatexFormatter(style=HeliosPygments, verboptions=",".join(options))

    return highlight(code, lex, formatter)


class HeliosCLexer(CLexer):
    name = "HeliosCLexer"
    EXTRA_KEYWORDS = [
        "HLS_INT",
        "HLS_INT32",
        "HLS_INT64",
        "HLS_REAL",
        "MPI_Comm",
        "HLS_Memory",
        "HLS_MemoryNode",
    ]

    def get_tokens_unprocessed(self, text):
        for index, token, value in CLexer.get_tokens_unprocessed(self, text):
            if token is Name and value in self.EXTRA_KEYWORDS:
                yield index, Keyword.Type, value
            else:
                yield index, token, value


class HeliosPygments(Style):
    default_style = ""
    blue = "#5679a3"
    orange = "#e59244"
    red = "#d1605d"
    lightblue = "#84b5b2"
    green = "#6a9f58"
    yellow = "#e8c960"
    purple = "#a87d9f"
    pink = "#f2a2a9"
    brown = "#977662"
    gray = "#b8b0ac"
    styles = {
        Comment: "italic " + gray,
        Keyword: "bold " + green,
        Keyword.Type: "bold " + red,
        Operator: "bold " + red,
        String: "italic " + blue,
        Number: "bold " + red,
        Name.Function: "bold " + blue,
    }


class DraculaPygments(Style):
    background_color = "#282a36"
    default_style = ""
    styles = {
        Comment: "#6272a4",
        Comment.Hashbang: "#6272a4",
        Comment.Multiline: "#6272a4",
        Comment.Preproc: "#ff79c6",
        Comment.Single: "#6272a4",
        Comment.Special: "#6272a4",
        Generic: "#f8f8f2",
        Generic.Deleted: "#8b080b",
        Generic.Emph: "#f8f8f2 underline",
        Generic.Error: "#f8f8f2",
        Generic.Heading: "#f8f8f2 bold",
        Generic.Inserted: "#f8f8f2 bold",
        Generic.Output: "#44475a",
        Generic.Prompt: "#f8f8f2",
        Generic.Strong: "#f8f8f2",
        Generic.Subheading: "#f8f8f2 bold",
        Generic.Traceback: "#f8f8f2",
        Error: "#f8f8f2",
        Keyword: "#ff79c6",
        Keyword.Constant: "#ff79c6",
        Keyword.Declaration: "#8be9fd italic",
        Keyword.Namespace: "#ff79c6",
        Keyword.Pseudo: "#ff79c6",
        Keyword.Reserved: "#ff79c6",
        Keyword.Type: "#8be9fd",
        Literal: "#f8f8f2",
        Literal.Date: "#f8f8f2",
        Name: "#f8f8f2",
        Name.Attribute: "#50fa7b",
        Name.Builtin: "#8be9fd italic",
        Name.Builtin.Pseudo: "#f8f8f2",
        Name.Class: "#50fa7b",
        Name.Constant: "#f8f8f2",
        Name.Decorator: "#f8f8f2",
        Name.Entity: "#f8f8f2",
        Name.Exception: "#f8f8f2",
        Name.Function: "#50fa7b",
        Name.Label: "#8be9fd italic",
        Name.Namespace: "#f8f8f2",
        Name.Other: "#f8f8f2",
        Name.Tag: "#ff79c6",
        Name.Variable: "#8be9fd italic",
        Name.Variable.Class: "#8be9fd italic",
        Name.Variable.Global: "#8be9fd italic",
        Name.Variable.Instance: "#8be9fd italic",
        Number: "#bd93f9",
        Number.Bin: "#bd93f9",
        Number.Float: "#bd93f9",
        Number.Hex: "#bd93f9",
        Number.Integer: "#bd93f9",
        Number.Integer.Long: "#bd93f9",
        Number.Oct: "#bd93f9",
        Operator: "#ff79c6",
        Operator.Word: "#ff79c6",
        Other: "#f8f8f2",
        Punctuation: "#f8f8f2",
        String: "#f1fa8c",
        String.Backtick: "#f1fa8c",
        String.Char: "#f1fa8c",
        String.Doc: "#f1fa8c",
        String.Double: "#f1fa8c",
        String.Escape: "#f1fa8c",
        String.Heredoc: "#f1fa8c",
        String.Interpol: "#f1fa8c",
        String.Other: "#f1fa8c",
        String.Regex: "#f1fa8c",
        String.Single: "#f1fa8c",
        String.Symbol: "#f1fa8c",
        Text: "#f8f8f2",
        Whitespace: "#f8f8f2",
    }



import os
import pyhelios

from typer import Typer, Argument, Option, Context
from typing import List, Optional

_help = f"{pyhelios.__description__} Command Line Interface (CLI)"
app = Typer(help=_help, epilog=f"Report bugs to {pyhelios.__maintainer__}")


@app.command()
def coverage(
    cov_file: str = Argument(..., metavar="", help="The coverage file"),
    output_dir: str = Argument(..., metavar="", help="LaTeX files output directory"),
):
    """Parse the coverage files and generate LaTeX files"""
    from pyhelios.coverage import Coverage

    Coverage(cov_file, output_dir)


@app.command()
def doxygen(
    xml_dir: str = Argument(..., metavar="", help="XML Doxygen output directory"),
    output_dir: str = Argument(..., metavar="", help="LaTeX files output directory"),
):
    """Parse XML Doxygen files and generate LaTeX files"""
    from pyhelios.documentation import Doxygen

    Doxygen(xml_dir, output_dir)


@app.command()
def plot(
    toml: str = Argument(..., metavar="*.toml", help="TOML configuration file"),
    tmp: str = Option(
        "", "--tmp", "-t", metavar="", help="LaTeX template file", show_default=False
    ),
    output_directory: str = Option(
        ".",
        "--output-directory",
        "-o",
        metavar="*",
        help="Figures output directory",
        show_default=False,
    ),
):
    """PUblication REady PLOT (PuRePlot)"""
    from pyhelios.plot import Plot

    Plot(toml, tmp, OutputDir=output_directory)


@app.command(
    context_settings={"allow_extra_args": True, "ignore_unknown_options": True}
)
def python(ctx: Context):
    """Helios Python interpreter"""
    from pyhelios.python import ipython

    ipython(ctx.args)


@app.command()
def render(
    template: str = Argument(..., metavar="*", help="Jinja2 template file"),
    toml: List[str] = Argument(..., metavar="*.toml", help="TOML configuration files"),
    output: str = Option(
        "render.out",
        "--output",
        "-o",
        metavar="*",
        help="Rendered output file",
        show_default=False,
    ),
):
    """Render a Jinja2 template file"""
    import flatdict
    import pyhelios.toml
    import pyhelios.template

    data = dict()
    for t in toml:
        d = pyhelios.toml.read(t)
        data = {**d, **data}
    data = flatdict.FlatDict(data, delimiter="_")
    pyhelios.template.render(template, output, dict(data))


@app.command()
def run(
    cmd: str = Argument(..., metavar="", help="Command"),
    stdin: Optional[List[str]] = Option(
        [], "--stdin", "-s", metavar="", help="Standard input list for the command"
    ),
    directory: str = Option(
        ".", "--dir", "-d", metavar="", help="Directory where is launched the command"
    ),
    shell: bool = Option(False, help="Shell execution"),
):
    """Run the command"""
    import pyhelios.exe

    stdout, stderr = pyhelios.exe.run(
        cmd, stdin="\n".join(stdin) + "\n", directory=directory, shell=shell
    )
    print(stderr) if stderr else print(stdout)


@app.command()
def version():
    """Show the version"""
    import pyhelios

    print(f"{pyhelios.__name__} {pyhelios.__version__}")


if __name__ == "__main__":
    app()



from warnings import filterwarnings
from traitlets.config import Config
from pygments.formatters import HtmlFormatter
from IPython import start_ipython
from IPython.terminal.prompts import Prompts
from IPython.terminal.prompts import Token
from pyhelios.highlight import DraculaPygments


class HeliosPrompt(Prompts):
    def in_prompt_tokens(self):
        return [
            (Token.Prompt, self.vi_mode()),
            (Token.Prompt, ">>> "),
            (Token.PromptNum, ""),
            (Token.Prompt, ""),
        ]

    def out_prompt_tokens(self):
        return [
            (Token.OutPrompt, ""),
            (Token.OutPromptNum, ""),
            (Token.OutPrompt, ""),
        ]


def ipython(args):
    filterwarnings("ignore")
    c = Config()
    c.InteractiveShellApp.exec_lines = ["import pyhelios"]
    c.InteractiveShell.colors = "Linux"
    c.InteractiveShell.separate_in = ""
    c.TerminalInteractiveShell.highlighting_style = HtmlFormatter(
        style=DraculaPygments
    ).style
    c.InteractiveShell.confirm_exit = False
    c.TerminalIPythonApp.display_banner = False
    c.TerminalInteractiveShell.highlighting_style_overrides = {
        Token.Prompt: "",
        Token.PromptNum: "",
        Token.OutPrompt: "",
        Token.OutPromptNum: "",
    }
    c.TerminalInteractiveShell.prompts_class = HeliosPrompt
    start_ipython(argv=args, config=c)



import os
import inspect
import pyhelios

from pyhelios.template import get_jinja_latex_template

HLS_LOW_MEDIUM_COVERAGE = 0.75
HLS_MEDIUM_HIGH_COVERAGE = 0.90

HLS_LOW_COVERAGE_COLOR = "red"
HLS_MEDIUM_COVERAGE_COLOR = "orange"
HLS_HIGH_COVERAGE_COLOR = "green"


class Coverage:
    def __init__(self, fn, output_dir):
        self.coverage = {}
        template = os.path.dirname(inspect.getfile(pyhelios))
        template = os.path.join(template, "share", "templates", "coverage.tex")
        self.template = get_jinja_latex_template(template)
        with open(fn, "r") as fid:
            self.fn = fid.read()
        for f in self.fn.split("end_of_record"):
            lines = f.split("\n")
            for l in lines:
                if l.startswith("SF:"):
                    file_name = os.path.basename(l.split("SF:")[-1])
                    mod_name = "hls_" + file_name.split("_")[1].split(".")[0]
                    file_name = file_name.replace("_", "\_")
                elif l.startswith("FNH:"):
                    cov_func = l.split("FNH:")[-1]
                elif l.startswith("FNF:"):
                    cov_func_tot = l.split("FNF:")[-1]
                elif l.startswith("LH:"):
                    cov_line = l.split("LH:")[-1]
                elif l.startswith("LF:"):
                    cov_line_tot = l.split("LF:")[-1]
            if not mod_name in self.coverage:
                self.coverage[mod_name] = []
            cov_line_percent = float(cov_line) / float(cov_line_tot)
            cov_func_percent = float(cov_func) / float(cov_func_tot)
            self.coverage[mod_name].append(
                [
                    file_name,
                    cov_line,
                    cov_line_tot,
                    cov_line_percent,
                    self.color(cov_line_percent),
                    cov_func,
                    cov_func_tot,
                    cov_func_percent,
                    self.color(cov_func_percent),
                ]
            )
        for mod_name in self.coverage:
            data = {
                "coverage": self.coverage[mod_name],
                "mod_name": mod_name.replace("_", "\_"),
            }
            fn = os.path.join(output_dir, mod_name + "__coverage.tex")
            with open(fn, "w") as fid:
                fid.write(self.template.render(data))

    def color(self, value):
        if value < HLS_LOW_MEDIUM_COVERAGE:
            return HLS_LOW_COVERAGE_COLOR
        elif value < HLS_MEDIUM_HIGH_COVERAGE:
            return HLS_MEDIUM_COVERAGE_COLOR
        else:
            return HLS_HIGH_COVERAGE_COLOR



import os
import sys
import inspect
import rich.pretty
from loguru import logger
from importlib.metadata import metadata

# -- Get the Metadata.
__version__ = metadata("pyhelios")["Version"]
__author__ = metadata("pyhelios")["Author"]
__maintainer__ = metadata("pyhelios")["Maintainer"]
__license__ = metadata("pyhelios")["License"]
__description__ = metadata("pyhelios")["Summary"]

# -- Configure the rich package.
rich.pretty.install()

# -- Configure le loguru package.
config = {
    "handlers": [
        {
            "sink": sys.stderr,
            "filter": lambda record: record["level"].name == "ERROR",
            "format": "<g>[{time:HH:mm:ss}]</g> <lvl>[{level}]</lvl> <lvl>{message}</lvl> <cyan>{name}</cyan>:<magenta>{line}</magenta>",
        },
        {
            "sink": sys.stdout,
            "filter": lambda record: record["level"].name == "INFO",
            "format": "<g>[{time:HH:mm:ss}]</g> <lvl>[{level}]</lvl> {message}",
        },
    ]
}
logger.configure(**config)
logger.level("ERROR", color="<red>")
logger.level("INFO", color="<green>")



import os
import inspect
import pyhelios
import xmltodict
import collections

from pyhelios.template import get_jinja_latex_template
from pyhelios.highlight import pygments_latex_render, pygments_latex_defines


def to_list(item):
    return [item] if isinstance(item, collections.OrderedDict) else item


class Define:
    def __init__(self, template, m):
        self.template = template
        self.prototype = "#define " + m["name"]
        if "param" in m:
            args = []
            m["param"] = to_list(m["param"])
            for p in m["param"]:
                args.append(p["defname"])
            self.prototype += "(" + ", ".join(args) + ")"

    def render(self):
        self.prototype = pygments_latex_render(self.prototype)
        return self.template.render(**self.__dict__)


class Typedef:
    def __init__(self, template, m):
        self.template = template
        self.prototype = m["definition"] + ";"

    def render(self):
        self.prototype = pygments_latex_render(self.prototype)
        return self.template.render(**self.__dict__)


class Structure:
    def __init__(self, template, s):
        self.template = template
        compounddef = s["compounddef"]
        self.prototype = "struct " + compounddef["compoundname"]
        self.members = []
        if compounddef["briefdescription"]:
            if compounddef["briefdescription"]["para"]:
                self.brief = compounddef["briefdescription"]["para"]
        if compounddef["detaileddescription"]:
            if compounddef["detaileddescription"]["para"]:
                self.long = compounddef["detaileddescription"]["para"]

        if compounddef["@kind"] == "struct":
            if "sectiondef" in compounddef:
                sectiondef = to_list(compounddef["sectiondef"])
                for s in sectiondef:
                    if s["@kind"] == "public-attrib":
                        memberdef = to_list(s["memberdef"])
                        for m in memberdef:
                            if m["@kind"] == "variable":
                                if m["detaileddescription"]:
                                    if m["detaileddescription"]["para"]:
                                        self.members.append(
                                            [
                                                m["definition"].replace(
                                                    compounddef["compoundname"] + "::",
                                                    "",
                                                ),
                                                m["name"].replace("_", "\_"),
                                                m["detaileddescription"][
                                                    "para"
                                                ].replace("_", "\_"),
                                            ]
                                        )

    def render(self):
        self.prototype += "{" + ", ".join([e[0] for e in self.members]) + "};"
        self.prototype = pygments_latex_render(self.prototype)
        return self.template.render(**self.__dict__)


class Function:
    def __init__(self, template, m):
        self.template = template
        self.prototype = m["definition"] + m["argsstring"] + ";"
        self.description = None
        if m["briefdescription"]:
            self.description = m["briefdescription"]["para"].replace("_", "\_")
        self.parameters = []
        if m["detaileddescription"]:
            if m["detaileddescription"]["para"]:
                parameterlist = m["detaileddescription"]["para"]["parameterlist"]
                if parameterlist["@kind"] == "param":
                    parameteritem = parameterlist["parameteritem"]
                    parameteritem = to_list(parameteritem)
                    for p in parameteritem:
                        self.parameters.append(
                            [
                                p["parameternamelist"]["parametername"][
                                    "#text"
                                ].replace("_", "\_"),
                                p["parameternamelist"]["parametername"][
                                    "@direction"
                                ].replace("_", "\_"),
                                p["parameterdescription"]["para"].replace("_", "\_"),
                            ]
                        )
        self.returns = None
        if m["detaileddescription"]:
            if m["detaileddescription"]["para"]:
                if "simplesect" in m["detaileddescription"]["para"]:
                    simplesect = m["detaileddescription"]["para"]["simplesect"]
                    if simplesect["@kind"] == "return":
                        self.returns = simplesect["para"].replace("_", "\_")

    def render(self):
        self.prototype = pygments_latex_render(self.prototype)
        return self.template.render(**self.__dict__)


class Doxygen:
    def __init__(self, xml_dir, output_dir):
        self.files = {}
        self.defines = {}
        self.typedefs = {}
        self.functions = {}
        self.structures = {}

        # -- LaTeX template.
        with open(os.path.join(output_dir, "pygments_defines.tex"), "w") as fid:
            fid.write(pygments_latex_defines())

        # -- Get the LaTeX template for defines.
        template = os.path.dirname(inspect.getfile(pyhelios))
        template = os.path.join(template, "share", "templates", "define.tex")
        self.define_template = get_jinja_latex_template(template)

        # -- Get the LaTeX template for typedefs.
        template = os.path.dirname(inspect.getfile(pyhelios))
        template = os.path.join(template, "share", "templates", "typedef.tex")
        self.typedef_template = get_jinja_latex_template(template)

        # -- Get the LaTeX template for structures.
        template = os.path.dirname(inspect.getfile(pyhelios))
        template = os.path.join(template, "share", "templates", "structure.tex")
        self.structure_template = get_jinja_latex_template(template)

        # -- Get the LaTeX template for functions.
        template = os.path.dirname(inspect.getfile(pyhelios))
        template = os.path.join(template, "share", "templates", "function.tex")
        self.function_template = get_jinja_latex_template(template)

        # -- Parse the Doxygen index.
        with open(os.path.join(xml_dir, "index.xml"), "r") as fid:
            doxygenindex = xmltodict.parse(fid.read())["doxygenindex"]
            for compound in doxygenindex["compound"]:
                if compound["@kind"] == "file":
                    xml = os.path.join(xml_dir, compound["@refid"] + ".xml")
                    self.files[compound["name"]] = xml

        # -- Parse the defines.
        for header, xml_file in self.files.items():
            self.defines[header] = []
            with open(xml_file, "r") as fid:
                doxygen = xmltodict.parse(fid.read())["doxygen"]
                compounddef = doxygen["compounddef"]
                if compounddef["@kind"] == "file":
                    if "sectiondef" in compounddef:
                        sectiondef = to_list(compounddef["sectiondef"])
                        for s in sectiondef:
                            if s["@kind"] == "define":
                                memberdef = to_list(s["memberdef"])
                                for m in memberdef:
                                    if m["@kind"] == "define":
                                        d = Define(self.define_template, m)
                                        self.defines[header].append(d.render())
            output_file = os.path.join(
                output_dir, os.path.splitext(header)[0] + "_define.tex"
            )
            with open(output_file, "w") as fid:
                for d in self.defines[header]:
                    fid.write(d)

        # -- Parse the typedefs.
        for header, xml_file in self.files.items():
            self.typedefs[header] = []
            with open(xml_file, "r") as fid:
                doxygen = xmltodict.parse(fid.read())["doxygen"]
                compounddef = doxygen["compounddef"]
                if compounddef["@kind"] == "file":
                    if "sectiondef" in compounddef:
                        sectiondef = to_list(compounddef["sectiondef"])
                        for s in sectiondef:
                            if s["@kind"] == "typedef":
                                memberdef = to_list(s["memberdef"])
                                for m in memberdef:
                                    if m["@kind"] == "typedef":
                                        t = Typedef(self.typedef_template, m)
                                        self.typedefs[header].append(t.render())
            output_file = os.path.join(
                output_dir, os.path.splitext(header)[0] + "_typedef.tex"
            )
            with open(output_file, "w") as fid:
                for t in self.typedefs[header]:
                    fid.write(t)

        # -- Parse the structures.
        for header, xml_file in self.files.items():
            self.structures[header] = []
            with open(xml_file, "r") as fid:
                doxygen = xmltodict.parse(fid.read())["doxygen"]
                compounddef = doxygen["compounddef"]
                if compounddef["@kind"] == "file":
                    if "innerclass" in compounddef:
                        innerclass = to_list(compounddef["innerclass"])
                        for ic in innerclass:
                            file_name = os.path.join(xml_dir, ic["@refid"] + ".xml")
                            with open(file_name, "r") as fid:
                                d = xmltodict.parse(fid.read())["doxygen"]
                                s = Structure(self.structure_template, d)
                                self.structures[header].append(s.render())
            output_file = os.path.join(
                output_dir, os.path.splitext(header)[0] + "_structure.tex"
            )
            with open(output_file, "w") as fid:
                for s in self.structures[header]:
                    fid.write(s)

        # -- Parse the functions.
        for header, xml_file in self.files.items():
            self.functions[header] = []
            with open(xml_file, "r") as fid:
                doxygen = xmltodict.parse(fid.read())["doxygen"]
                compounddef = doxygen["compounddef"]
                if compounddef["@kind"] == "file":
                    if "sectiondef" in compounddef:
                        sectiondef = to_list(compounddef["sectiondef"])
                        for s in sectiondef:
                            if s["@kind"] == "func":
                                memberdef = to_list(s["memberdef"])
                                for m in memberdef:
                                    if m["@kind"] == "function":
                                        f = Function(self.function_template, m)
                                        self.functions[header].append(f.render())
            output_file = os.path.join(output_dir, os.path.splitext(header)[0] + ".tex")
            with open(output_file, "w") as fid:
                for f in self.functions[header]:
                    fid.write(f)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyhelios']

package_data = \
{'': ['*'], 'pyhelios': ['share/plot/*', 'share/templates/*']}

install_requires = \
['cmake-format>=0.6.0,<0.7.0',
 'flatdict>=4.0.0,<5.0.0',
 'ipython>=7.0.0,<8.0.0',
 'jinja2>=3.0.0,<4.0.0',
 'loguru==0.5.3',
 'numpy>=1.21.0,<2.0.0',
 'pygments>=2.0.0,<3.0.0',
 'pytomlpp>=1.0.0,<2.0.0',
 'rich>=10.0.0,<11.0.0',
 'traitlets>=5.0.0,<6.0.0',
 'typer>=0.4.0,<0.5.0',
 'xmltodict>=0.12.0,<0.13.0']

entry_points = \
{'console_scripts': ['pyhls = pyhelios.pyhls:app']}

setup_kwargs = {
    'name': 'pyhelios',
    'version': '2.0.5',
    'description': "The Helios' toolbox",
    'long_description': "# pyhelios: the Helios' toolbox\n\n## Installation\n\n```sh\npip install pyhelios\n```\n\n## Contributing\n\nThe pyhelios Python package is managed by awesome tools: [poetry](https://python-poetry.org/) and [tox](https://tox.wiki/en/latest/index.html).\n\n1. Create a virtual environment using poetry.\n\n```sh\npoetry env use `which python`\n```\n\n2. Build the pyhelios Python package (sdist and wheel).\n\n```sh\npoetry build\n```\n\n2. Install in the previously created virtual environment.\n\n```sh\npoetry install\n```\n\n3. Run the tests.\n\n```sh\npoetry run python -m tox\n```\n\n4. Publish to the [Python Package Index](https://pypi.org/)\n\n```sh\npoetry publish\n```\n\nIf necessary, [poetry](https://python-poetry.org/) can bump the version for you.\nThe new version should be a valid [semver](https://semver.org/) string or a valid bump rule: patch, minor, major, prepatch, preminor, premajor, prerelease.\n\n```sh\npoetry version patch[minor, major, prepatch, preminor, premajor, prerelease]\n```",
    'author': 'Julien Vanharen',
    'author_email': 'julien.vanharen@gmail.com',
    'maintainer': 'Julien Vanharen',
    'maintainer_email': 'julien.vanharen@gmail.com',
    'url': 'https://pypi.org/project/pyhelios',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.0,<4.0.0',
}


setup(**setup_kwargs)
